@@ -10,14 +10,19 @@ An overly-simple CBOR packer/unpacker.
 from __future__ import annotations
 
 # Typing
-from moat.lib.codec import Extension, NoCodecError
+from moat.lib.codec import Extension
 from moat.lib.codec.cbor import Codec, Tag
-from moat.lib.codec.proxy import DProxy, Proxy, name2obj, obj2name, unwrap_obj, wrap_obj, get_proxy
+from moat.lib.codec.proxy import DProxy, Proxy, get_proxy, name2obj, obj2name, unwrap_obj, wrap_obj
 
 from . import NotGiven
 from .path import Path
 
-__all__ = ["std_ext", "StdCBOR"]
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from moat.lib.codec import ByteType
+
+__all__ = ["StdCBOR", "std_ext"]
 
 std_ext = Extension()
 
@@ -30,11 +35,12 @@ class StdCBOR(Codec):
     def __init__(self):
         super().__init__(ext=std_ext)
 
-    def decode(self, data:bytes):
-        if data == b'':
+    def decode(self, data: ByteType):  # noqa:D102,RUF100
+        if data == b"":
             return NotGiven
         return super().decode(data)
 
+
 Codec = StdCBOR
 
 
@@ -42,7 +48,7 @@ Codec = StdCBOR
 def _enc_dpr(codec, obj):
     codec  # noqa:B018
     res = [obj.name] + obj.a
-    if obj.k or res and isinstance(res[-1], dict):
+    if obj.k or (res and isinstance(res[-1], dict)):
         res.append(obj.k)
     return res
 
@@ -77,7 +83,7 @@ def _dec_obj(codec, val):
 @std_ext.encoder(39, Path)
 def _enc_path(codec, val):
     codec  # noqa:B018
-    return val.raw
+    return val.raw_rooted
 
 
 @std_ext.decoder(39)
@@ -85,7 +91,7 @@ def _dec_path(codec, val):
     codec  # noqa:B018
     if not isinstance(val, (list, tuple)):
         return Tag(39, val)  # not decodable
-    return Path.build(val)
+    return Path.build(val, decoded=True)
 
 
 @std_ext.encoder(None, object)
