@@ -1,21 +1,21 @@
 # Trio step monitors
 
 from __future__ import annotations
-from trio.abc import Instrument
-from trio import Cancelled
-from trio._core._instrumentation import Instruments
+
 import sys
-from outcome import Value, Error
+
+from trio._core._instrumentation import Instruments
+from trio.abc import Instrument
 
 
 class Instr(Instrument):
     exited = None
 
     def before_task_step(self, task):
-        print("BEF:", task, task._next_send, file=sys.stderr)
+        print("BEF:", task, task._next_send, file=sys.stderr)  # noqa: SLF001
 
     def after_task_step(self, task):
-        print("AFT:", task, task._next_send, file=sys.stderr)
+        print("AFT:", task, task._next_send, file=sys.stderr)  # noqa: SLF001
 
     def task_exited(self, task, outcome=None):
         print("EXIT", task, outcome, file=sys.stderr)
@@ -23,7 +23,7 @@ class Instr(Instrument):
 
 def hookup(runner=None):
     if runner is None:
-        import inspect as i
+        import inspect as i  # noqa: PLC0415
 
         f = i.currentframe().f_back
         while "runner" not in f.f_locals:
