@@ -4,12 +4,13 @@ This module contains various helper functions and classes for dictionaries.
 
 from __future__ import annotations
 
-from collections.abc import Mapping
 from copy import deepcopy
 
 from . import NotGiven
 
-__all__ = ["combine_dict", "drop_dict", "attrdict", "to_attrdict"]
+from collections.abc import Mapping, MutableSequence
+
+__all__ = ["attrdict", "combine_dict", "drop_dict", "to_attrdict"]
 
 
 def combine_dict(*d, cls=dict, deep=False) -> dict:
@@ -43,6 +44,8 @@ def combine_dict(*d, cls=dict, deep=False) -> dict:
     for kv in d:
         if kv is None:
             continue
+        if not isinstance(kv, dict):
+            raise TypeError
         for k, v in kv.items():
             if k not in keys:
                 keys[k] = []
@@ -168,13 +171,13 @@ class attrdict(dict):
                 v.append(value)
                 return val
             if px >= len(v):
-                if px>len(v)+10:
+                if px > len(v) + 10:
                     raise ValueError(f"Won't pad the array (want {px}, has {len(v)}).")
-                v.extend([NotGiven]*(1 + px - len(v)))
+                v.extend([NotGiven] * (1 + px - len(v)))
             v[px] = value
         elif not isinstance(v, Mapping):
-            raise ValueError((v,px))
-        elif px in v:
+            raise ValueError((v, px))
+        elif px in v and isinstance(v[px], (Mapping, MutableSequence)):
             v[px] = combine_dict(value, v[px], cls=type(self))
         else:
             v[px] = value
