@@ -9,8 +9,6 @@ import logging
 import logging.config
 import os
 import sys
-from collections import defaultdict
-from collections.abc import Mapping
 from contextlib import suppress
 from contextvars import ContextVar
 from functools import partial, wraps
@@ -18,11 +16,14 @@ from pathlib import Path as FSPath
 
 import asyncclick as click
 
+from . import NotGiven
+from .config import CFG, ensure_cfg
 from .dict import attrdict, to_attrdict
 from .exc import ungroup
-from . import NotGiven
 from .merge import merge
-from .config import CFG, ensure_cfg
+
+from collections import defaultdict
+from collections.abc import Mapping
 
 try:
     from moat.lib.codec.proxy import Proxy
@@ -35,22 +36,21 @@ from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
     from typing import Awaitable  # noqa:UP035
-    from asyncclick import command
 
 logger = logging.getLogger("_loader")
 
 __all__ = [
-    "main_",
-    "read_cfg",
-    "load_cfg",
-    "wrap_main",
     "Loader",
-    "load_subgroup",
-    "list_ext",
-    "load_ext",
     "attr_args",
-    "process_args",
+    "list_ext",
+    "load_cfg",
+    "load_ext",
+    "load_subgroup",
+    "main_",
     "option_ng",
+    "process_args",
+    "read_cfg",
+    "wrap_main",
 ]
 
 this_load = ContextVar("this_load", default=None)
@@ -59,7 +59,7 @@ NoneType = type(None)
 
 
 def _no_config(*a, **k):  # noqa:ARG001
-    import warnings
+    import warnings  # noqa: PLC0415
 
     warnings.warn("Call to logging config ignored", stacklevel=2)
 
@@ -76,27 +76,26 @@ def attr_args(
     """
     Add an option for setting possibly-hierarchical values to `click.command`.
 
-    New behavior (default): use ``-s``/``--set`` with prefix-tagged values:
+    ``-s``/``--set`` accepts these prefix-tagged values:
     * ~str
     * =value (``=-``/``=t``/``=f``/``=n`` for ``undefined``/`True`/`False`/`None`)
-    * .path (except for a single ``:`` for an empty path)
-    * :named_proxy
+    * .path (the leading dot is stripped)
+    * :path (the colon is not stripped)
+    * ^named_proxy
 
     These are enabled (rather, displayed) by passing ``True`` in
     ``with_val``, ``with_eval``, ``with_path``, and ``with_proxy``,
     respectively. The latter defaults to `False`, all others to `True`.
 
-    Old behavior (hidden in new mode): Adds separate
+    Legacy behavior (hidden unless `with_combined` is False): Adds separate
     ``-v``/``--var``, ``-e``/``--eval``, -p``/``--path, and
     ``-P``/``--proxy`` arguments.
 
-    Use `attr_args(with_path=False)` to skip path arguments. Ditto for
-    `with_eval`.
-
-    Use ``with_combined=False`` to get the old behavior.
+    Use ``with_combined=False`` to get legacy behavior.
     Use ``with_combined=LETTER`` to change the default from ``-s``.
 
-    Old-behavior long options are availabile, but hidden, in "new" mode.
+    In new mode, Legacy short options are not availabile;
+    long options are available but hidden.
     """
 
     def _proc(proc):
@@ -107,9 +106,9 @@ def attr_args(
             if with_eval:
                 ht.append("=value")
             if with_path:
-                ht.append(".path")
+                ht.append(".path, :path")
             if with_proxy:
-                ht.append(":name")
+                ht.append("^proxy")
             ht = "|".join(ht)
 
             args = ("--set",) + (
@@ -127,7 +126,9 @@ def attr_args(
                 hidden=not with_eval or not with_combined,
             )(proc)
 
-        args = ("--path",) + (("-p",) if with_path else ())
+        args = (f"--{with_path}" if isinstance(with_path, str) else "--path",) + (
+            ("-p",) if with_path else ()
+        )
         proc = click.option(
             *args,
             "path_",
@@ -138,7 +139,9 @@ def attr_args(
             hidden=not with_path or not with_combined,
         )(proc)
 
-        args = ("--eval",) + (("-e",) if with_eval else ())
+        args = (f"--{with_eval}" if isinstance(with_eval, str) else "--eval",) + (
+            ("-e",) if with_eval is True else ()
+        )
         proc = click.option(
             *args,
             "eval_",
@@ -149,7 +152,9 @@ def attr_args(
             hidden=not with_eval or not with_combined,
         )(proc)
 
-        args = ("--var",) + (("-v",) if with_var else ())
+        args = (f"--{with_var}" if isinstance(with_var, str) else "--var",) + (
+            ("-v",) if with_var is True else ()
+        )
         proc = click.option(
             *args,
             "vars_",
@@ -160,7 +165,9 @@ def attr_args(
             hidden=not with_var or not with_combined,
         )(proc)
 
-        args = ("--proxy",) + (("-P",) if with_proxy else ())
+        args = (f"--{with_proxy}" if isinstance(with_proxy, str) else "--proxy",) + (
+            ("-P",) if with_proxy is True else ()
+        )
         proc = click.option(
             *args,
             "proxy_",
@@ -216,11 +223,11 @@ def process_args(val, set_=(), vars_=(), eval_=(), path_=(), proxy_=(), no_path=
                 v = None
             elif v[0] == "=":
                 v = eval(v[1:])  # pylint: disable=W0631
-            elif v == ":":
-                v = P(":")
+            elif v[0] == ":":
+                v = P(v)
             elif v[0] == ".":
                 v = P(v[1:])
-            elif v[0] == ":":
+            elif v[0] == "^":
                 v = Proxy(v[1:])
             else:
                 try:
@@ -366,7 +373,7 @@ def load_cfg(name):
 
 
 def _namespaces(name):
-    import pkgutil  # pylint: disable=import-outside-toplevel
+    import pkgutil  # pylint: disable=import-outside-toplevel  # noqa: PLC0415
 
     try:
         ext = importlib.import_module(name)
@@ -441,7 +448,7 @@ def load_subgroup(
     ext_pre=None,
     ext_post=None,
     **kw,
-) -> Command:
+) -> click.Command:
     """
     A decorator like click.group, enabling loading of subcommands
 
@@ -582,7 +589,7 @@ class Loader(click.Group):
         logger.debug("* List: %s.*.%s / %s.*.%s", sub_pre, sub_post, ext_pre, ext_post)
 
         if sub_pre:
-            logger.debug("Adding sub %s",sub_pre)
+            logger.debug("Adding sub %s", sub_pre)
             for _finder, name, _ispkg in _namespaces(sub_pre):
                 # ruff:noqa:PLW2901 # var overwritten
                 name = name.rsplit(".", 1)[1]
@@ -592,7 +599,7 @@ class Loader(click.Group):
                     rv.append(name)
 
         if ext_pre:
-            logger.debug("Adding ext %s",ext_pre)
+            logger.debug("Adding ext %s", ext_pre)
             for n, _ in list_ext(ext_pre):
                 if load_ext(ext_pre, n, *ext_post, err=ctx.obj.debug_loader):
                     rv.append(n)
@@ -696,9 +703,9 @@ async def main_(ctx, verbose, quiet, help=False, **kv):  # pylint: disable=redef
         return
     ctx._moat_invoked = True  # pylint: disable=protected-access
     wrap_main(ctx=ctx, verbose=max(0, 1 + verbose - quiet), **kv)
-    if help or ctx.invoked_subcommand is None and not ctx.protected_args:
+    if help or (ctx.invoked_subcommand is None and not ctx.args and not ctx._protected_args):
         print(ctx.get_help())
-        ctx.exit()
+        await ctx.aexit()
 
 
 def wrap_main(  # pylint: disable=redefined-builtin,inconsistent-return-statements
@@ -780,7 +787,7 @@ def wrap_main(  # pylint: disable=redefined-builtin,inconsistent-return-statemen
         opts["name"] = name
 
     if sub_pre is True:
-        import inspect  # pylint: disable=import-outside-toplevel
+        import inspect  # pylint: disable=import-outside-toplevel  # noqa: PLC0415
 
         sub_pre = inspect.currentframe().f_back.f_globals["__package__"]
     elif sub_pre is None:
@@ -837,17 +844,19 @@ def wrap_main(  # pylint: disable=redefined-builtin,inconsistent-return-statemen
         merge(cfg, read_cfg(name, fn), replace=False)
 
     obj.cfg = cfg = to_attrdict(cfg)
+    if cfg.env.in_test:
+        cfg.env.in_test(cfg)
 
     obj.debug = verbose
     obj.DEBUG = debug
 
     # Don't forget to import toplevel config files
-    import moat
+    import moat  # noqa: PLC0415
 
     try:
         p = moat.__path__
     except AttributeError:
-        p = (str(FSPath(ext.__file__).parent),)
+        p = (str(FSPath(moat.__file__).parent),)
     for fp in p:
         try:
             with (FSPath(fp) / "_cfg.yaml").open("r") as f:
@@ -867,7 +876,7 @@ def wrap_main(  # pylint: disable=redefined-builtin,inconsistent-return-statemen
     if wrap:
         pass
     elif hasattr(logging.root, "_MoaT"):
-        logging.debug("Logging already set up")
+        logging.debug("Logging already set up")  # noqa:LOG015
     else:
         # Configure logging. This is a somewhat arcane art.
         lcfg = obj.cfg.setdefault("logging", {})
@@ -941,5 +950,5 @@ def _ng(type_):
     return gen
 
 
-def option_ng(*a, type=str, **kw):
+def option_ng(*a, type=str, **kw):  # noqa: A002, D103
     return click.option(*a, **kw, type=_ng(type), default=NotGiven)
