@@ -12,7 +12,7 @@ else:  # pragma: no cover
     except ImportError:
         import random
     else:
-        random = tcr._r
+        random = tcr._r  # noqa: SLF001
 
 # Intended to be unambiguous, no special characters
 al_unique = "bcdfghjkmnpqrstvwxyzBCDFGHJKMNPQRSTVWXYZ23456789"
@@ -26,7 +26,7 @@ al_ascii = bytes(x for x in range(33, 127) if x != 92).decode("ascii")
 # lowercase letters only
 al_az = "abcdefghijklmnopqrstuvwxyz"
 
-__all__ = ["id2str", "gen_ident", "al_unique", "al_lower", "al_ascii", "al_az"]
+__all__ = ["al_ascii", "al_az", "al_lower", "al_unique", "gen_ident", "id2str"]
 
 
 def gen_ident(k=10, /, *, alphabet=al_unique):
@@ -46,8 +46,6 @@ def id2str(k, /, *, alphabet=al_unique):
     res = []
     la = len(alphabet)
     while k:
-        res.append(alphabet[k%la])
+        res.append(alphabet[k % la])
         k //= la
-    return ''.join(res)
-
-
+    return "".join(res)
