@@ -1,5 +1,5 @@
 """
-This module contains various helper functions and classes.
+This module contains the systemd service helper.
 """
 
 from __future__ import annotations
@@ -16,6 +16,31 @@ except ImportError:
     notify = None
 
 
+class RunMsg(anyio.abc.TaskStatus):
+    """A helper to signal readiness or status updates to systemd.
+
+    It also duck-types as :class:`anyio.abc.TaskStatus.`
+    """
+
+    def __init__(self, tg, obj):
+        self.tg = tg
+        self.obj = obj
+        self.evt = anyio.Event()
+
+    def set(self):  # pylint:disable=missing-function-docstring
+        self.evt.set()
+        if notify is not None:
+            notify("READY=1")
+        if self.obj is not None and self.obj.debug:
+            print("Running.")
+
+    def started(self, value: anyio.abc.T_Contra | None = None):
+        "mock task_status.started"
+        if value is not None:
+            raise ValueError("value is ignored")
+        self.set()
+
+
 @asynccontextmanager
 async def as_service(obj=None):
     """
@@ -25,8 +50,8 @@ async def as_service(obj=None):
     Arguments:
         obj: command context. Needs a ``debug`` attribute.
 
-    The CM yields a (duck-typed) event whose ``set`` method will
-    trigger a ``READY=1`` mesage to systemd.
+    The ACM yields a (duck-typed) event whose ``set`` method will
+    trigger the ``READY=1`` mesage to systemd.
     """
 
     async def run_keepalive(usec):
@@ -41,28 +66,11 @@ async def as_service(obj=None):
         pid = os.getpid()
         epid = int(os.environ.get("WATCHDOG_PID", pid))
         if pid == epid:
-            return int(os.environ.get("WATCHDOG_USEC", 0))
+            # prevent the watchdog from restarting / running concurrently
+            os.environ["WATCHDOG_PID"] = "0"
+            return int(os.environ.get("WATCHDOG_USEC", "0"))
         return 0
 
-    class RunMsg:
-        """A fake event that signals readiness"""
-
-        def __init__(self, tg, obj):
-            self.tg = tg
-            self.obj = obj
-
-        def set(self):  # pylint:disable=missing-function-docstring
-            if notify is not None:
-                notify("READY=1")
-            if self.obj is not None and self.obj.debug:
-                print("Running.")
-
-        def started(self, data=None):
-            "mock task_status.started"
-            if data is not None:
-                raise ValueError("data is ignored")
-            self.set()
-
     async with anyio.create_task_group() as tg:
         usec = need_keepalive()
         if usec:
