@@ -6,12 +6,13 @@ from __future__ import annotations
 
 import os
 import sys
-from collections.abc import Mapping, Sequence
 
 import ruyaml as yaml
 
 from .dict import attrdict
 
+from collections.abc import Mapping, Sequence
+
 try:
     from moat.lib.codec.proxy import DProxy, Proxy
 except ImportError:
@@ -21,13 +22,13 @@ except ImportError:
 from .path import Path
 
 __all__ = [
-    "yload",
-    "yprint",
-    "yformat",
-    "yaml_repr",
-    "yaml_parse",
     "add_repr",
     "load_ansible_repr",
+    "yaml_parse",
+    "yaml_repr",
+    "yformat",
+    "yload",
+    "yprint",
 ]
 
 SafeRepresenter = yaml.representer.SafeRepresenter
@@ -40,9 +41,9 @@ SafeRepresenter.add_representer(attrdict, SafeRepresenter.represent_dict)
 
 def load_ansible_repr():
     "Call me if you're using `moat.util` in conjunction with Ansible."
-    from ansible.parsing.yaml.objects import AnsibleUnicode
-    from ansible.utils.unsafe_proxy import AnsibleUnsafeText
-    from ansible.vars.hostvars import HostVars, HostVarsVars
+    from ansible.parsing.yaml.objects import AnsibleUnicode  # noqa: PLC0415
+    from ansible.utils.unsafe_proxy import AnsibleUnsafeText  # noqa: PLC0415
+    from ansible.vars.hostvars import HostVars, HostVarsVars  # noqa: PLC0415
 
     SafeRepresenter.add_representer(HostVars, SafeRepresenter.represent_dict)
     SafeRepresenter.add_representer(HostVarsVars, SafeRepresenter.represent_dict)
@@ -103,8 +104,9 @@ def _proxy_repr(dumper, data):
     # return ScalarNode(tag, value, style=style)
     # return yaml.events.ScalarEvent(anchor=None, tag='!P', implicit=(True, True), value=str(data))
 
+
 def _dproxy_repr(dumper, data):
-    return dumper.represent_scalar("!DP", repr([data.name,data.a,data.k]))
+    return dumper.represent_scalar("!DP", repr([data.name, data.a, data.k]))
     # return ScalarNode(tag, value, style=style)
     # return yaml.events.ScalarEvent(anchor=None, tag='!P', implicit=(True, True), value=str(data))
 
@@ -113,7 +115,8 @@ def read_env(loader, node):
     value = loader.construct_scalar(node)
     return os.environ[value]
 
-def _el_repr(dumper, data):
+
+def _el_repr(dumper, data):  # noqa: ARG001
     return dumper.represent_scalar("!R", "...")
 
 
@@ -130,7 +133,10 @@ if DProxy is not None:
 
 
 def _name2obj(constructor, node):
-    from moat.lib.codec.proxy import name2obj
+    constructor  # noqa:B018
+
+    from moat.lib.codec.proxy import name2obj  # noqa: PLC0415
+
     return name2obj(node.value)
 
 
@@ -230,7 +236,7 @@ def yformat(data, compact=None):
     :param stream: the file to write to, defaults to stdout.
     :param compact: Write single lines if possible, default False.
     """
-    from io import StringIO  # pylint: disable=import-outside-toplevel
+    from io import StringIO  # pylint: disable=import-outside-toplevel  # noqa: PLC0415
 
     s = StringIO()
     yprint(data, compact=compact, stream=s)
